%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%SOCAT_make_mat_file.m					%
%							%
%Loads data from Benjamin, merges with external		%
%salinity, pressure, bathymetry & puts into a .mat file	%
%							%
%Are Olsen, Aug_2006					%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clearvars -except cruises c_i


%%%%%%%%%%%%%%%USER defined settings%%%%%%%%%%%%%%

%data files location
rawDataLoc='/Users/are/arbeid/SOCAT/Ver2Code/test_files_new/';

%location of ncep slps
ncepDataLoc = '/Users/are/fra_geos1/benjamin/slps/';

%location of woa salinities
woaDataLoc = '/Users/are/fra_geos1/benjamin/WOA05_sss/s0112an1.nc';

%location of bathymery
%batLoc='/Users/are/fra_geos1/benjamin/bathymetry/ETOPO2v2c_f4.nc';
%NEW VERSION; ABOVE NOT USED, INSTEAD MAPPING TOOLBOX ETOPO2 FUNCTION
%place ETOPO2v2c.hdf in same dir as this script


%mat files location
matLoc='/Users/are/arbeid/SOCAT/Ver2Code/matData/';

%%%%%%%%%%%%%%%end of USER defined settings%%%%%%%%%%%%%%

%Read the file into cell arrays
form_str=cat(2,'%s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t',...
		'%s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t',...
		'%s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t');

[sta type mon day yr hh mm ss long lat bottomD depth temp sal  ...
 xCO2_water_sst_dry_ppm xCO2_water_equi_temp_dry_ppm...
 fCO2_water_sst_100humidity_uatm...
 fCO2_water_equi_uatm...
 pCO2_water_sst_100humidity_uatm pCO2_water_equi_temp....
 Temperature_equi Pressure_atm...
 Pressure_equi wind_direc_true wind_direc_rel wind_speed_true wind_speed_rel ship_speed ship_direc Humidity xCO2_atm fCO2_atm...
 pCO2_atm dfCO2 dpCO2]...
=textread(cat(2,eval('rawDataLoc'),...
eval('cruises{c_i}'),'_imp.1.txt'),eval('form_str'), 'headerlines',6, 'emptyvalue', NaN);
  
%convert cells to doubles for calculations, these are preceded by a "C_"
A=who
 
 
for i = 1:size(A,1)
 	C=A(i);
 	v=genvarname(strcat('C_',char(C)));
 	eval([v '=char(eval(char(A(i))));']);
 	eval([v '=str2num(eval(v));']);
end

%merging tends to crash if lat or lon is zero, thus use almost zero in those cases
%NB,m zero is written to file anyway. 
index_zeros=find(C_long==0); C_long(index_zeros)=0.001;
index_zeros2=find(C_lat==0); C_lat(index_zeros2)=0.001;
%%%%%%%%%%%%%%%%merge with NCEP/NCAR slp

time_num=datenum(C_yr,C_mon,C_day,C_hh,C_mm, C_ss);	%hrs since 1-1-00 00:00;
relevantYears =unique(C_yr);

for j = 1: length(relevantYears);
	index=find(C_yr==relevantYears(j));
	str=cat(2,eval('ncepDataLoc'),'slp.', num2str(relevantYears(j)),'.nc');
	%SLP=netcdf.open(eval('str'), 'nowrite');
   
    %SLP=netcdf(eval('str'), 'nowrite');

    Lat_slp=ncread(str, 'lat');
	Lon_slp=ncread(str, 'lon');
 
	Lat_slp(1)=mean(Lat_slp(1:2));	%remove singulatities in grid.
	Lat_slp(end)=mean(Lat_slp(end-1:end));
	
    %deal with time, NCEP/NCAR units is: 
    %ans =hours since 1-1-1 00:00:0.0
    
    pre_time_slp=ncread(str, 'time');
    NCEP_base_time=24.*(datenum('0001-01-01 00:00:00', 'yyyy-mm-dd HH:MM:SS')-2);
    time_slp=(NCEP_base_time+pre_time_slp)./24;
	
    %and SLP
 	slp = ncread(str, 'slp');
	
    %scale & offset
	%  slp=slp.*1+ 119765;
	slp=slp./100;		%to hectopascals
	
    %slp=shiftdim(slp,1);

		%do the merging
		%find indecies of closest lat/lon in NCEP grid
		
		[TLON TLAT]=meshgrid(Lon_slp, Lat_slp);
		[sTX, sTY]=size(TLON);
		IND=lonlatind(TLON, TLAT, C_long(index), C_lat(index));
		
		[I,J]=ind2sub([sTX sTY], IND);

		%find indecies of closest times
		[dataTime ncepTIME]=meshgrid(time_num(index), time_slp);
		diffs=abs(dataTime-ncepTIME);
		[minVal, indexTime]=min(diffs);
		
		%get the data
		for k=1:length(I)
			slp_out(index(k))=slp(J(k),I(k), indexTime(k));
		end	
end
ncep_slp=slp_out';
%clean up
clear SLP Lat_slp Lon_slp t1 time_slp t3 t4 t5 t6 slp X Y TLON TLAt sTX sTY IND I J dataTime ncepTIME diffs minVAL slp_out

%%%%%%%%%%%%%%%%merge with WOA salinity

sss=ncread(eval('woaDataLoc'), 's0112an1'); %a 360x180x24x12 variable
%sss=reshape(sss,[180 360 24 12]);
lat_sss=ncread(eval('woaDataLoc'), 'lat');
lon_sss=ncread(eval('woaDataLoc'), 'lon');

for s_i=1:12;				%for each month
	s_i
	index=find(C_mon==s_i);
	sss_month=squeeze(sss(:, :, 1, s_i)); %convert to 2D, 180x360
    sss_month=shiftdim(sss_month, 1); %reshape to 180x360

	%merge
	lat_i=C_lat(index);		%find monthly positions
	lon_i=C_long(index);
					%find indices
		[TLON TLAT]=meshgrid(lon_sss(:), lat_sss(:));
		[sTX, sTY]=size(TLON);
		[IND]=lonlatind(TLON, TLAT, lon_i, lat_i);
					%get the data
		sss_out(index)=sss_month(IND);
end
woa_sss=sss_out';

%for WOA cells labelled land, use default salinity 35 & set WOA land marker to 1, 0 otherwise. 
%The effect of salinty on the fCO2 calculations is regardless minimal

%For xCO2 = 300 and press = 1000
%At 40C, changing salinity from 0 -40 changes fCO2 by 0.5
%at 0C, it changes by 0.03
%at 15C it changes by 0.01 when SSS is changed from 34 to 35.

woa_land_marker=zeros(length(woa_sss),1);
index_land=find(woa_sss>1000); 
woa_sss(index_land)=35;
woa_land_marker(index_land)=1;

%clean up
clear SSS sss lat_sss lon_sss index sss_month lat_i lon_i TLON TLAT sTZ sTY IND I J sss_out

%%%%%%%Get bathymetry from m_tbase
C_longETOPO=C_long;
C_longETOPO(C_long>180)=C_long(C_long>180)-360;

for i = 1: length(C_long)
    z_out(i)=etopo(1, [C_lat(i) C_lat(i)], [C_longETOPO(i) C_longETOPO(i)]);
end

z_out=z_out';
%%%%%%%%%%%%%%%%merge with ETOPO2 bathymetry%%%%%%%%%%%%
% z=ncread(eval('batLoc'),'z'); 
% z=shiftdim(z,1); %reshape
% lat_z=ncread(eval('batLoc'),'y'); 
% lon_z=ncread(eval('batLoc'),'x'); 
% 
% %index=find(lon_z<0);
% %lon_z(index)=lon_z(index)+360;
% 
% [TLON TLAT]=meshgrid(lon_z(:),lat_z(:));
% [sTX, sTY]=size(TLON);
% keyboard
% [INDz]=lonlatind(TLON, TLAT, C_long, C_lat);
% z_out=z(INDz);
% 
% %clean up
% clear BAT z lat_z lon_z index TLON TLAT sTX sTY INDz

%%%%%%%%save%%%%%%%%%%

save_str=cat(2, 'save ', eval('matLoc') ,eval('cruises{c_i}'),'.mat')

eval(save_str)


